/////////////////////////////////////////////////////////////////////////////
///
/// \file
///
/// A program for the computation of simplicial
/// algebraic topological model.
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2009-2016 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started on March 24, 2009. Last revision: October 21, 2013.


// include the string hashing definitions (before including "hashsets.h")
#include "chaincon/stringhash.h"

// include selected header files from the CHomP library
#include "chomp/system/config.h"
#include "chomp/struct/hashsets.h"

// include relevant local header files
#include "chaincon/simplex.h"
#include "chaincon/emptycell.h"
#include "atmodmain.h"


// --------------------------------------------------
// -------------------- OVERTURE --------------------
// --------------------------------------------------

/// The title of the program and licensing information.
const char *title = "\
Simplicial homology computation with an algebraic topological model.\n\
Version 0.05 (Sep 24, 2015). Copyright (C) 1997-2016 by Pawel Pilarczyk.\n\
This is free software. No warranty. Consult 'license.txt' for details.";

/// Brief help information on the program's usage.
const char *helpinfo = "\
This program computes simplicial homology groups and homology generators.\n\
The ring of coefficients is Z or Z_p (the integers modulo prime number p).\n\
Call with:\n\
filename - the name of a file that contains a list of simplices,\n\
Switches and additional arguments:\n\
filename2 - the name of an additional file for relative (co)homology;\n\
-pN - selection of the coefficients: p=0 for Z, p=1 for purely\n\
\tcombinatorial procedures, p > 1 for Z_p (default: p=1); use p <= 181.\n\
-r - compute reduced homology (with the empty set as a cell of dim -1).\n\
-b - don't add boundary cells (use -bx and -ba for X and A selectively),\n\
-dpi, -dincl, -dphi - display the computed maps: pi, incl, phi,\n\
-drepr - display homology representants (meaningful for -aN with N < 3),\n\
-aN - homology algorithm: 0 = old (very slow), 1 = new without additional\n\
optimization (relatively fast), 2 = new (default), 3,4 = using the SNF.\n\
--verify - do additional verification of the computed maps,\n\
--log filename - save the output to a file (without progress indicators),\n\
--quiet - suppress data output to the screen (whcih can be still logged),\n\
--help - display this brief help information only and exit.\n\
For more information please consult the accompanying documentation\n\
or ask the program's author at http://www.PawelPilarczyk.com/.";


// --------------------------------------------------
// ---------------------- main ----------------------
// --------------------------------------------------

/// The main procedure of the program.
/// Returns: 0 = Ok, -1 = Error, 1 = Help displayed, 2 = Wrong arguments.
int main (int argc, char *argv [])
{
	return atModelMain<tSimplex<int_t,SettableEmptyCell> >
		(argc, argv, title, helpinfo);
} /* main */

