/////////////////////////////////////////////////////////////////////////////
///
/// \file
///
/// A program for the computation of the Alexander-Whitney diagonal
/// for cubical complexes, using Serre's formula.
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2009-2016 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started on March 24, 2009. Last revision: January 6, 2013.


// include the hashing definitions (before including "hashsets.h")
#include "chaincon/stringhash.h"
#include "chaincon/pair.h"

// include selected header files from the CHomP library
#include "chomp/system/config.h"
#include "chomp/struct/hashsets.h"

// include relevant local header files
#include "chaincon/cubcell.h"
#include "chaincon/emptycell.h"
#include "chaincon/wrapping.h"
#include "chaincon/awdiagcub.h"
#define SPACE_WRAPPING
#include "awdiag2dmain.h"


// --------------------------------------------------
// -------------------- OVERTURE --------------------
// --------------------------------------------------

/// The title of the program and licensing information.
const char *title = "\
Alexander-Whitney diagonal for cubical complexes (Serre's formula).\n\
Version 0.02 (Sep 24, 2015). Copyright (C) 1997-2016 by Pawel Pilarczyk.\n\
This is free software. No warranty. Consult 'license.txt' for details.";

/// Brief help information on the program's usage.
const char *helpinfo = "\
This program computes the Alexander-Whitney diagonal for 2-dimensional\n\
homology generators, and represents the result in terms of 1-dimensional\n\
homology genrators. In other words, this program decomposes cavities\n\
into products of cycles.\n\
The ring of coefficients is set to Z_2, the integers modulo 2.\n\
Call with:\n\
filename - the name of a file that contains a list of cubical cells,\n\
Switches and additional arguments:\n\
filename2 - the name of an additional file for relative homology;\n\
-ddiag - display the A-W diagonal of each 2D homology generator,\n\
-aN - homology algorithm: 0 = old (very slow), 1 = new without additional\n\
\toptimization (relatively fast), 2 = new (default), 3 = using the SNF.\n\
-r - compute reduced homology (with the empty set as a cell of dim -1),\n\
-b - don't add boundary cells (use -bx and -ba for X and A selectively),\n\
-w N1,..,Nk - set space wrapping, a.k.a. periodic boundary conditions;\n\
\trepeat the n-tuple for each dimension n; use 0 for no wrapping,\n\
--verify - do additional verification of the computed A-W diagonal,\n\
--log filename - save the output to a file (without progress indicators),\n\
--quiet - suppress data output to the screen (whcih can be still logged),\n\
--help - display this brief help information only and exit.\n\
For more information please consult the accompanying documentation\n\
or ask the program's author at http://www.PawelPilarczyk.com/.";


// --------------------------------------------------
// ---------------------- main ----------------------
// --------------------------------------------------

/// The main procedure of the program.
/// Returns: 0 = Ok, -1 = Error, 1 = Help displayed, 2 = Wrong arguments.
int main (int argc, char *argv [])
{
	return awdiag2dMain<tCubCell<int_t,SettableWrapping<int_t>,
		SettableEmptyCell> > (argc, argv, title, helpinfo);
} /* main */

