/////////////////////////////////////////////////////////////////////////////
///
/// \file
///
/// A version of the Alexander-Whitney diagonal for general simplicial sets.
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2009-2016 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started on March 24, 2009. Last revision: December 3, 2014.


#ifndef _CHAINCON_AWDIAGSIMGEN_H_
#define _CHAINCON_AWDIAGSIMGEN_H_


// include some standard C++ header files
#include <istream>
#include <ostream>

// include selected header files from the CHomP library
#include "chomp/system/config.h"

// include relevant local header files
#include "chaincon/combtensor.h"
#include "chaincon/tensor.h"
#include "chaincon/ez_aw.h"


// --------------------------------------------------
// ------------------ AW diagonal -------------------
// --------------------------------------------------

/// Computes the Alexander-Whitney diagonal of a simplicial cell.
template <class SimCellT>
inline void AWdiagonal (const SimCellT &s,
	tCombTensor<SimCellT,SimCellT> &t)
{
	EZ_AW (s, s, t);
	return;
} /* AWdiagonal */

/// Computes the Alexander-Whitney diagonal of a simplicial cell.
template <class SimCellT, class CoefT>
inline void AWdiagonal (const SimCellT &s,
	tTensor<SimCellT,SimCellT,CoefT> &t)
{
	EZ_AW (s, s, t);
	return;
} /* AWdiagonal */


#endif // _CHAINCON_AWDIAGSIMGEN_H_

