/////////////////////////////////////////////////////////////////////////////
///
/// \file
///
/// A function that creates a filter from a set of cells
/// of different dimensions.
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2009-2016 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started on March 24, 2009. Last revision: August 21, 2013.


#ifndef _CHAINCON_SETFILTER_H_
#define _CHAINCON_SETFILTER_H_


// include relevant local header files

// include selected header files from the CHomP library
#include "chomp/system/config.h"

// include some standard C++ header files


// --------------------------------------------------
// ----------------- set to filter ------------------
// --------------------------------------------------

/// Transforms a collection of cells to a filter by adding them
/// in the order of decreasing dimension.
/// The maximal dimension of cells of interest must be provided.
template <class SetT, class FilterT>
inline void set2filter (const SetT &set, FilterT &filter, int maxDim)
{
	using chomp::homology::sbug;

	int_t size = set. size ();
	for (int dim = maxDim; dim >= 0; -- dim)
	{
		for (int_t i = 0; i < size; ++ i)
		{
			if (set [i]. dim () == dim)
				filter. add (set [i]);
		}
	}

	return;
} /* set2filter */


#endif // _CHAINCON_SETFILTER_H_

