/////////////////////////////////////////////////////////////////////////////
///
/// \file
///
/// A program for the computation of the Steenrod squares,
/// using algebraic minimal models for cubical sets
/// and the direct formula by Marek Krcal.
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2009-2016 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started on July 4, 2013. Last revision: January 23, 2015.


// the choice of cubes or simplices
#ifndef CUBES_NOT_SIMPLICES
#define CUBES_NOT_SIMPLICES 1
#endif

// do not use the direct formula by Marek Krcal for cubes
#ifndef DIRECT_CUBICAL_FORMULA
#define DIRECT_CUBICAL_FORMULA 1
#endif

// include some standard C++ header files
#include <istream>
#include <ostream>
#include <string>

// include the string hashing definitions (before including "hashsets.h")
#include "chaincon/stringhash.h"

// include selected header files from the CHomP library
#include "chomp/system/config.h"
#include "chomp/struct/hashsets.h"

// include relevant local header files
#include "chaincon/ssquarescub.h"
#include "ssqmain.h"


// --------------------------------------------------
// -------------------- OVERTURE --------------------
// --------------------------------------------------

/// The title of the program and licensing information.
const char *title = "\
Computation of Steenrod squares using an Algebraic Minimal Model.\n\
Version 0.01 (Feb 6, 2016). Copyright (C) 1997-2016 by Pawel Pilarczyk.\n\
This is free software. No warranty. Consult 'license.txt' for details.";

/// Brief help information on the program's usage.
const char *helpinfo = "\
This program computes and displays all the non-zero Steenrod squares\n\
of a cubical set. The program uses a formula introduced by Marek Krcal\n\
for a cubical complex, without subdivision into a simplicial set.\n\
Note that the ring of coefficients is fixed to Z_2.\n\
Call with:\n\
filename - the name of a file that contains a list of cells for the test,\n\
Switches and additional arguments:\n\
filename2 - the name of an additional file for relative (co)homology;\n\
-r - compute reduced homology (with the empty set as a cell of dim -1),\n\
-b - don't add boundary cells (use -bx and -ba for X and A selectively),\n\
--log filename - save the output to a file (without progress indicators),\n\
--quiet - suppress data output to the screen (whcih can be still logged),\n\
--help - display this brief help information only and exit.\n\
For more information please consult the accompanying documentation\n\
or ask the program's author at http://www.PawelPilarczyk.com/.";


// --------------------------------------------------
// ---------------------- main ----------------------
// --------------------------------------------------

/// The main procedure of the program.
/// Returns: 0 = Ok, -1 = Error, 1 = Help displayed, 2 = Wrong arguments.
int main (int argc, char *argv [])
{
	return steenrodSquaresMain (argc, argv, title, helpinfo);
} /* main */

