#!/usr/bin/perl

#############################################################################
#
# This script creates a suspension of a simplicial complex.
#
#############################################################################

# Copyright (C) 2015-2016 by Pawel Pilarczyk.
#
# This file is part of my research software package. This is free software:
# you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file "license.txt". If not,
# please, see <http://www.gnu.org/licenses/>.
#
# Started on September 17, 2015. Last revision: January 19, 2016.

use strict;

print "SuspSimpl 0.01 (C) 2015-2016 by Pawel Pilarczyk.\n";
die "Call with the INPUT and OUTPUT file names.\n" if (scalar (@ARGV) < 2);

my $inName = $ARGV [0];
my $outName = $ARGV [1];

my $file;
my @simplices;
my $highestVertexNumber;
open ($file, "<$inName") or die "Can't open '${inName}'.\n";
print "Reading simplices from '${inName}...\n";
my $count = 0;
while (my $line = <$file>)
{
	next unless $line =~ m/^\s*\(([\d,\s]+)\)\s*$/;
	my $simplex = $1;
	push (@simplices, $simplex);
	my @vertices = split (/\s*,\s*/, $simplex);
	foreach my $vertexNumber (@vertices)
	{
		$highestVertexNumber = $vertexNumber
			if (!defined ($highestVertexNumber) ||
			($highestVertexNumber < $vertexNumber));
	}
	++ $count;
}
close $file;
print "${count} simplices read.\n";
print "The highest vertex number found is ${highestVertexNumber}.\n";

open ($file, ">$outName") or die "Can't create '${outName}'.\n";
print "Writing the suspension to '${outName}'...\n";
for (my $add = 1; $add <= 2; ++ $add)
{
	my $vertex = $highestVertexNumber + $add;
	foreach my $simplex (@simplices)
	{
		print $file "(${simplex},$vertex)\n";
	}
	print $file "\n";
}
print "Done.\n";

